#include<stdio.h>
#include<stdlib.h>
#include<unistd.h>

/* holds information about a file and where it should be placed in the image */
struct Snippet
{
	char fname[1024]; /* bad coding, but I need this to work fast */
	int startrow;
	int endrow;
	int startcol;
	int endcol;
};

struct Snippet* SnarfDataFromPerlScript(FILE *in, int numimages);
void LoadSnippetIntoImage(struct Snippet *snip, unsigned char *image, int rows,
	int columns);

int main(void)
{
	char version[10];
	int rows = 0;
	int columns = 0;
	int range = 0;
	int numimages = 0;
	struct Snippet *snips;
	FILE *infile = stdin;
	unsigned char *image;
	char name[1024];
	int i;
	FILE *outfile;

	/* snarf all of the header information */
	fscanf(infile, "Version: %s\n", version);
	fscanf(infile, "Rows: %d\n", &rows);
	fscanf(infile, "Columns: %d\n", &columns);
	fscanf(infile, "Range: %d\n", &range);
	fscanf(infile, "Name: %s\n", name);
	fscanf(infile, "NumImages: %d\n", &numimages);

	printf("Version: %s\n", version);
	printf("Rows: %d\n", rows);
	printf("Columns: %d\n", columns);
	printf("Range: %d\n", range);
	printf("Name: %s\n", name);
	printf("NumImages: %d\n", numimages);

	/* read in the imformation about the data */
	snips = SnarfDataFromPerlScript(infile, numimages);

	/* this is the destination image for everything */
	image = 
		(unsigned char *)malloc(sizeof(unsigned char) * (rows * (columns * 3)));
	if (image == NULL)
	{
		printf("Out of memory, Sorry.\n");
		exit(EXIT_FAILURE);
	}
	memset(image, 0, rows * (columns * 3));

	for (i = 0; i < numimages; i++)
	{
		LoadSnippetIntoImage(&snips[i], image, rows, columns);
	}

	outfile = fopen(name, "w+");
	if (outfile == NULL)
	{
		printf("Could not open %s\n", name);
		exit(EXIT_FAILURE);
	}
	fprintf(outfile, "%s\n%d %d\n%d\n", version, columns, rows, range);

	fwrite(image, rows * (columns*3), 1, outfile);
	fclose(outfile);
	
	free(image);
	free(snips);

	return(0);
}

struct Snippet* SnarfDataFromPerlScript(FILE *infile, int numimages)
{
	struct Snippet *snips;
	int i;

	snips = (struct Snippet*)malloc(sizeof(struct Snippet) * numimages);
	if (snips == NULL)
	{
		printf("Out of memory. Sorry.\n");
		exit(EXIT_FAILURE);
	}

	for (i = 0; i < numimages; i++)
	{
		fscanf(infile, "%s %d %d %d %d\n", snips[i].fname, &snips[i].startrow,
			&snips[i].endrow, &snips[i].startcol, &snips[i].endcol);
	}

	return snips;
}

void LoadSnippetIntoImage(struct Snippet *snip, unsigned char *image, int rows,
	int columns)
{
	FILE *file;
	char junk[1024];
	int i;
	int fpos;

	file = fopen(snip->fname, "r");
	if (file == NULL)
	{
		printf("Skipping file: %s\n", snip->fname);
		return;
	}

	/* read the version, sizes, and range */
	if ((fgets(junk, 1024, file) == NULL) ||
		(fgets(junk, 1024, file) == NULL) ||
		(fgets(junk, 1024, file) == NULL))
	{
		printf("Read 0 rows out of %d in file: %s\n", 
			snip->endrow - snip->startrow, snip->fname);
			fclose(file);
		return;
	}

	fpos = ftell(file); /* start of the binary data */

	/* Thar be magic here. Read from the right place out of the file into
		the right place in the image. Stuff is multiplied by 3 because there
		are red, green, and blue components in the ppm. Also, if the
		file is partially done, just bail out having loaded whatever was up to
		the end of file. */

	for (i = 0; i < snip->endrow - snip->startrow; i++)
	{
		fseek(file, fpos + i*(3*columns) + (snip->startcol*3), SEEK_SET);
		if (fread(&image[(snip->startcol*3) + ((snip->startrow + i)*3*columns)],
			(snip->endcol - snip->startcol)*3, 1, file) <= 0)
		{
			printf("Read %d rows out of %d in file: %s\n", 
				i, snip->endrow - snip->startrow, snip->fname);
			fclose(file);
			return;
		}
	}

	fclose(file);
}











